/*******************************************************************************
 * Copyright (c) 2010- Association for Decentralized Information Management in
 * Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.example;

import org.simantics.databoard.accessor.DoubleAccessor;
import org.simantics.databoard.accessor.RecordAccessor;
import org.simantics.databoard.accessor.reference.AccessorReference;
import org.simantics.databoard.binding.mutable.Variant;
import org.simantics.databoard.datasource.DatasourceUtil;
import org.simantics.databoard.datasource.ExampleDatasource;
import org.simantics.databoard.datasource.StepwiseDatasourceAccessor;

public class DatasourceExample {

	public static void main(String[] args) throws Exception {
		
		// Create datasource. See ExampleDatasource
		ExampleDatasource datasource = new ExampleDatasource();
		
		// Open accessor to datasource
		StepwiseDatasourceAccessor accessor = new StepwiseDatasourceAccessor( datasource );
		
		// Get root id constant
		Variant rootId = DatasourceUtil.ROOT_ID;
		
		// Get children of root
		Variant children[] = DatasourceUtil.getChildren(accessor, rootId);
		
		// Print children, and grand children
		for (Variant nodeId : children) {
			System.out.println( nodeId );
			Variant children2[] = DatasourceUtil.getChildren(accessor, nodeId);
			for (Variant nodeId2 : children2)
				System.out.println("  o " + nodeId2 );
		}

		
		// Open accessor to Sine variable
		RecordAccessor sine = accessor.getAccessor( AccessorReference.parseString("nodes/SSINE") );
		// Open accessor to the value 
		DoubleAccessor sineValue = accessor.getAccessor( AccessorReference.parseString("nodes/SSINE/value/o/v") );
		
		// Step and read value 10 times
		for (int i=0; i<10; i++) {
			double y = sineValue.getValue();
			System.out.println("Step "+i+": sine="+y);
			datasource.step( null );
		}
	
		accessor.dispose();
	}
	
}

