/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.accessor.reference;


/**
 * A reference by index to the value of an union. 
 *
 * "u-<index>"
 *
 * @author Toni Kalajainen <toni.kalajainen@vtt.fi>
 */
public class TagIndexReference extends AccessorReference {

	public int tag; 
	
	public TagIndexReference(int tagIndex) 
	{
		tag = tagIndex;
	}
	
	public TagIndexReference(int tagIndex, AccessorReference child) 
	{
		super(child);
		tag = tagIndex;
	}
	
	@Override
	public String toString(boolean labelReference) {
		return labelReference ? ""+tag : "u-"+tag;
	}
	
	@Override
	public AccessorReference clone() {
		return new TagIndexReference(tag, childReference==null ? null : childReference.clone());
	}	
	
	@Override
	public boolean equals(Object obj) {
		if (obj instanceof TagIndexReference == false) return false;
		TagIndexReference other = (TagIndexReference) obj;
		if (other.tag != tag) return false;
		if (other.hasChildReference() != hasChildReference()) return false;
		if (hasChildReference() && !other.childReference.equals(childReference)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int hash = 42388823 + 37 * tag;
		if (hasChildReference()) hash = 31*hash + childReference.hashCode();		
		return hash;
	}	
	
}

