/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.binding.mutable;

import java.util.Set;

import org.simantics.databoard.Bindings;
import org.simantics.databoard.adapter.AdaptException;
import org.simantics.databoard.binding.Binding;
import org.simantics.databoard.binding.VariantBinding;
import org.simantics.databoard.binding.error.BindingException;
import org.simantics.databoard.type.DataType;

/**
 * VariantContainerBinding binds VariantType to {@link Variant} Class. 
 */
public class VariantContainerBinding extends VariantBinding {
		
	public static VariantContainerBinding INSTANCE = new VariantContainerBinding();
	
	VariantContainerBinding() {}
	
	@Override
	public boolean isImmutable() {
		return false;
	}
	
	@Override
	public Object create(Object value, Binding binding) throws BindingException {
		Variant result = new Variant();
		result.binding = binding;
		result.value = value;
		return result;
	}
	
	@Override
	public Binding getBinding(Object variant) throws BindingException {
		Variant result = (Variant) variant;
		return result.binding;
	}
	
	@Override
	public DataType getType(Object variant) throws BindingException {
		Variant result = (Variant) variant;
		return result.binding.type();
	}
	
	@Override
	public Object getValue(Object variant, Binding binding) throws BindingException {
		Variant var = (Variant) variant;
		try {
			Binding binding1 = var.binding;
			Object value1 = var.value;
			Binding binding2 = binding;
			Object value2 = Bindings.adapt(value1, binding1, binding2);
			return value2;
		} catch (AdaptException e) {
			throw new BindingException(e);
		}
	}
	
	public Object getValue(Object variant) throws BindingException {
		Variant var = (Variant) variant;
		return var.value;
	}

	@Override
	public void setValue(Object variant, Object value, Binding binding)
			throws BindingException {
		assert(isValid(binding, value));
		Variant var = (Variant) variant;
		var.binding = binding;
		var.value = value;
	}

	@Override
	public void assertInstaceIsValid(Object obj, Set<Object> validInstances) throws BindingException {
		if (obj==null) throw new BindingException("null value is not Variant");
		if (obj instanceof Variant == false) throw new BindingException("wrong class, VariantContainer expected");
		Variant var = (Variant) obj;
		if (var.binding==null) throw new BindingException("Binding is expected");		
		var.binding.assertInstaceIsValid(var.value, validInstances);
	}

	@Override
	public boolean isInstance(Object obj) {
		if (obj==null) return false;
		if (obj instanceof Variant == false) return false;
		return true;
	}
	
	boolean isValid(Binding binding, Object obj) throws BindingException {
		binding.assertInstaceIsValid(obj);
		return true;
	}
	
}
