/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.bindingscheme;

import java.util.HashMap;
import java.util.Map;

import org.simantics.databoard.Bindings;
import org.simantics.databoard.binding.Binding;
import org.simantics.databoard.binding.error.BindingConstructionException;
import org.simantics.databoard.type.DataType;

/**
 * Binding Repository stores datatype to binding mappings. 
 * It does not allow 
 *
 * @author Toni Kalajainen <toni.kalajainen@vtt.fi>
 */
public class BindingRepository implements BindingScheme {

	Map<DataType, Binding> map = new HashMap<DataType, Binding>();
	
	@Override
	public synchronized Binding getBinding(DataType type)
	{		
		return map.get(type);
	}

	@Override
	public synchronized boolean supportsType(DataType type) 
	{		
		return map.containsKey(type);
	}
	
	/**
	 * Add a binding. If a binding for the data type already exists,
	 * the previous binding is removed.
	 * 
	 * @param binding
	 */
	public synchronized void add(Binding binding)
	{
		map.put(binding.type(), binding);
	}
	
	public void add(Class<?> clazz)
	throws BindingConstructionException
	{
		Binding binding = Bindings.getBinding(clazz);
		synchronized(this) {
			map.put(binding.type(), binding);
		}
	}

}

