/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.tests;

import java.awt.geom.Rectangle2D;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import org.simantics.databoard.Bindings;
import org.simantics.databoard.DataTypes;
import org.simantics.databoard.accessor.java.JavaLong;
import org.simantics.databoard.annotations.KeyType;
import org.simantics.databoard.annotations.Optional;
import org.simantics.databoard.annotations.Referable;
import org.simantics.databoard.annotations.ValueType;
import org.simantics.databoard.binding.Binding;
import org.simantics.databoard.binding.IntegerBinding;
import org.simantics.databoard.binding.LongBinding;
import org.simantics.databoard.binding.MapBinding;
import org.simantics.databoard.binding.RecordBinding;
import org.simantics.databoard.binding.java.ArrayListBinding;
import org.simantics.databoard.binding.java.IntegerBindingDefault;
import org.simantics.databoard.binding.java.LongBindingDefault;
import org.simantics.databoard.binding.java.TreeMapBinding;
import org.simantics.databoard.binding.mutable.Variant;
import org.simantics.databoard.parser.repository.DataTypeRepository;
import org.simantics.databoard.type.ArrayType;
import org.simantics.databoard.type.DataType;
import org.simantics.databoard.type.LongType;

public class Jotakin2 {
	
	
	
	public static class Foo {
		public TreeMap<Long, String> stringSampleMap;
		
		public @KeyType(Long.class) @ValueType(Integer[].class)
		TreeMap<Long, Integer[] > integerArraySampleMapA;
//		public @KeyType(Long.class) @ValueType(ArrayList.class)
//		TreeMap<Long, ArrayList<Integer> > integerArraySampleMapB;
		
		Long helper_key;
		ArrayList<Integer> helper_value;

		public Map<String, Integer> x;
		public HashMap<String, Integer> a;
		public TreeMap<String, Integer> b;
		
		public Set<String> q;
		public HashSet<String> w;
		public TreeSet<String> e;
		
		public List<Double> y;
		
		public TreeMap<Long, Boolean> booleanSampleMap;
		public TreeMap<Long, Byte> byteSampleMap;
		public TreeMap<Long, Integer> shortSampleMap;// databoard  does not support Short, special handling in write method
		public TreeMap<Long, Integer> integerSampleMap;
		public TreeMap<Long, Long> longSampleMap;
		public TreeMap<Long, Float> floatSampleMap;
		public TreeMap<Long, Double> doubleSampleMap;
	}

//	public interface HelloMBean { 
//		 
//	    public void sayHello() throws Exception; 
//	    public int add(int x, int y); 
//	 
//	    public String getName(); 
//	 
//	    public int getCacheSize(); 
//	    public void setCacheSize(int size); 
//	}
//	
//	public static class Hello implements HelloMBean {
//
//		@Override
//		public int add(int x, int y) {
//			// TODO Auto-generated method stub
//			return 0;
//		}
//
//		@Override
//		public int getCacheSize() {
//			// TODO Auto-generated method stub
//			return 0;
//		}
//
//		@Override
//		public String getName() {
//			// TODO Auto-generated method stub
//			return null;
//		}
//
//		@Override
//		public void sayHello() 
//		throws Exception {
//			System.out.println("Hello");
////			throw new Exception("BUU");
//		}
//
//		@Override
//		public void setCacheSize(int size) {
//			// TODO Auto-generated method stub
//			
//		} 
//	}
//	
//	public interface MyInterface {
//		void hello();
//	}
//	

//	public static class FEMMesh {
//		Topology topology;
//		MeshData[] data;
//	}
	
	public static class Topology {
		
	}
	
//	public static class Vertex
//	
//	public abstract static class MeshData {
//		public static class VertexData 
//	}
//	
//	
	
	@Referable public static class MyRecord {
		
		String id;
		
		@Optional String name;
		
		double x, y, z;
		
		@Optional MyRecord linkToNext;
		
	}
	
	public static class FOOBAR {
		public Variant x;
		public Variant y;
	}
	
	public static class Foo1 {
		public int id;
	}
	
	public static class Foo2 {
		public int id;
		public @Optional String name;
	}
	
	@Referable public static class Node {
		public FOOBAR foobar;
		public int value;
		public Node next;
	}
	
	public static @Referable class Fooo {
		public Fooo f;
	}
	
	public enum Cars { Volvo, Audi, Mercedes, Nissan }
	
	
	static public class ComponentContent {}
	
	static public @Referable class Component {
		 public int id;      // Apros module id
		 public String name;
		 public ComponentContent content;
		 
		 public Component(int id, String name, ComponentContent content) {
		  this.id = id;
		  this.name = name;
		  this.content = content;
		 }
		}
	
	public static void main(String[] args) throws Exception {
		System.out.println(Map.class.isAssignableFrom(TreeMap.class));
		System.out.println(TreeMap.class.isAssignableFrom(Map.class));
		
		RecordBinding bb = Bindings.getBinding(Foo.class);
		System.out.println(bb.type());
		
		
		// Long
		Binding longBinding = LongBindingDefault.INSTANCE;		
		// ArrayList<Long>
		Binding arrayListLongBinding = new ArrayListBinding( new ArrayType(longBinding.type()), longBinding );		

		// Integer
		Binding intBinding = IntegerBindingDefault.INSTANCE;		
		// ArrayList<Integer>
		Binding arrayListIntegerBinding = new ArrayListBinding( new ArrayType(intBinding.type()), intBinding );
		// ArrayList<ArrayList<Integer>>
		Binding arrayListArrayListIntegerBinding = new ArrayListBinding( new ArrayType( arrayListIntegerBinding.type() ), arrayListIntegerBinding );
		
		// TreeMap<ArrayList<Long>, ArrayList<ArrayList<Integer>>> 
		MapBinding mb = new TreeMapBinding( arrayListLongBinding, arrayListArrayListIntegerBinding );
		
		System.out.println(mb.type());
		
		
		RecordBinding b = (RecordBinding) Bindings.getBinding(Component.class);
		Component c = (Component) b.createPartial();
		System.out.println(b.printValueDefinition(c, true));
		
		
		Foo1 foo1 = new Foo1();
		Foo2 foo2 = (Foo2) Bindings.adapt(foo1, Bindings.getBinding(Foo1.class), Bindings.getBinding(Foo2.class));
		System.out.println(Bindings.toString(foo2));
		
		
		System.out.println(Bindings.toString("X"));
		System.out.println(Bindings.toString(5.0));
		System.out.println(Bindings.toString( new Rectangle2D.Double(100,100,200,200)));
		
		System.out.println(DataTypes.getDataType(Exception.class));
		
		System.out.println( DataTypes.getDataType(Foo.class) );
		
		System.out.println(DataTypes.getDataType("DataType"));
		
		DataType type = DataTypes.getDataType( Cars.class );
		String typedef = type.toSingleLineString();
		System.out.println(typedef);
		
		DataTypes.addDefinitions("type Cars = | Volvo | Audi | Mercedes | Nissan");
		DataTypes.addDefinitions("type Cars =  | Volvo {}  | Audi {}  | Mercedes {}  | Nissan {}");
		DataTypes.addDefinitions("type Cars =  | Volvo {}  | Audi {}  | Mercedes {}  | Nissan {}");
		
		
		Charset UTF8        = Charset.forName("UTF-8");
		DataTypeRepository repo = DataTypes.datatypeRepository;

		
			
		
// Equals ongelma - ei ole korjattu //
//		
//		Binding bb = Bindings.getBindingUnchecked(Fooo.class);
//
//		Fooo a = new Fooo();
//		Fooo b = new Fooo();
//		Fooo c = new Fooo();
//		a.f = b; 
//		b.f = a;
//		c.f = c;
//		
//		System.out.println( bb.equals(a, b) );
//		System.out.println( bb.equals(b, a) );
//		System.out.println( bb.equals(a, c) );
//		System.out.println( bb.equals(c, c) );
//
//		System.out.println( bb.hashValue(a) );
//		System.out.println( bb.hashValue(b) );
//		System.out.println( bb.hashValue(c) );
//		
//		System.out.println( DataTypes.getDataType( Range.class ) );
//		System.out.println( DataTypes.getDataType( Limit.class ) );
		
//		
//		DataType type;
//		Binding binding;
//		Object o;
//		
//		binding = Bindings.getBinding( IntegerType.INSTANCE );
//		Binding b2 = Bindings.getBinding( LongType.INSTANCE );
//		Adapter a = Bindings.getTypeAdapter(binding, b2);
//		System.out.println(System.identityHashCode(a));
//		System.gc();
//		
//		a = Bindings.getTypeAdapter(binding, b2);
//		System.out.println(System.identityHashCode(a));
//		a = null;
//		System.gc();
//		
//		a = Bindings.getTypeAdapter(binding, b2);
//		System.out.println(System.identityHashCode(a));
//		
//		
//		binding = Bindings.getBinding( new IntegerType("m") );
//		System.out.println(System.identityHashCode(binding));
//		System.gc();
//
//		System.out.println(System.identityHashCode( Bindings.getBinding( new IntegerType("m") )));		
//		binding = null;
//		System.gc();
//		
//		System.out.println(System.identityHashCode( Bindings.getBinding( new IntegerType("m") ) ));
//		System.gc();
		
		
		/*
		VariantType vt1 = VariantType.INSTANCE;
		VariantType vt2 = new VariantType();
		VariantType vt3 = (VariantType) DataTypes.getDataType(Object.class);
		
		binding = Bindings.getBinding( Variant.class );
		
		binding = Bindings.getBinding( ContainerReference.class );
		System.out.println(binding.getDataType());
		
		binding = Bindings.getBinding( Event.class );
		System.out.println(binding.getDataType());
		
		binding = Bindings.getBinding( new VariantType() );
		for (int i=0; i<1000; i++) {
			o = binding.createRandomUnchecked(i);
			binding.assertInstaceIsValid(o, null);				
			System.out.print(i+": ");
			System.out.println(binding.printValue(o, true));
		}
		
		*/
		
		/*			
		binding = Bindings.getBinding(MutableBoolean.class);
		type = binding.getDataType();
		o = binding.createDefault();
		System.out.println(type +", "+ o.getClass() +", "+ binding.printValue(o, true));		
		
		binding = Bindings.getBinding(MutableByte.class);
		type = binding.getDataType();
		o = binding.createDefault();
		System.out.println(type +", "+ o.getClass() +", "+ binding.printValue(o, true));		
		
		binding = Bindings.getBinding(MutableInteger.class);
		type = binding.getDataType();
		o = binding.createDefault();
		System.out.println(type +", "+ o.getClass() +", "+ binding.printValue(o, true));		
		
		binding = Bindings.getBinding(MutableLong.class);
		type = binding.getDataType();
		o = binding.createDefault();
		System.out.println(type +", "+ o.getClass() +", "+ binding.printValue(o, true));		
		
		binding = Bindings.getBinding(MutableFloat.class);
		type = binding.getDataType();
		o = binding.createDefault();
		System.out.println(type +", "+ o.getClass() +", "+ binding.printValue(o, true));		
		
		binding = Bindings.getBinding(MutableDouble.class);
		type = binding.getDataType();
		o = binding.createDefault();
		System.out.println(type +", "+ o.getClass() +", "+ binding.printValue(o, true));		
		
		binding = Bindings.getBinding(MutableString.class);
		type = binding.getDataType();
		o = binding.createDefault();
		System.out.println(type +", "+ o.getClass() +", "+ binding.printValue(o, true));		
		
		binding = Bindings.getBinding(Node.class);
		type = binding.getDataType();
		o = binding.createDefault();
		System.out.println(type.toSingleLineString() +", "+ o.getClass() /* +", "+ binding.printValue(o, true));				
		
		*/
		
/*		
		VariantBinding vb = (VariantBinding) Bindings.getBinding(Variant.class);
		RecordBinding rb = (RecordBinding) Bindings.getBinding(FOOBAR.class);
		System.out.println( rb.getDataType().toSingleLineString() );
		FOOBAR fuu = new FOOBAR();
		fuu.x = new Variant();
		fuu.x.binding = StringJavaBinding.INSTANCE;
		fuu.x.value = "Hello";
		fuu.y = fuu.x;		
		System.out.println( rb.printValue(fuu, true) );
		
		ArrayBinding b = new ArrayListBinding(new ArrayType(VariantType.INSTANCE), Bindings.getBindingUnchecked(Object.class));
		Object l = b.create();
		b.add(l, "0");
		b.add(l, "1");
		b.add(l, "2");
		b.add(l, "3");
		b.add(l, "4");
		b.add(l, "5");
		b.add(l, "6");
		b.add(l, "7");
		b.remove(l, 6, 2);
		for (int i=0; i<b.size(l); i++)
			System.out.println(b.get(l, i));
*/		
		
/*
		String[] sa = new String[] {"s", "f"};
		Object[] oa = (Object[]) sa;
		for (int i=0; i<oa.length; i++)
			System.out.println(oa[i]);
	
		DataType dt = DataTypes.getDataType(Foo.class);		
		System.out.println(dt.toSingleLineString());
		Binding fooB = Bindings.getBinding(Foo.class);
		Foo value = (Foo) fooB.createDefaultUnchecked();
		value.e.add("Hei");
		value.e.add("Abu");
		System.out.println(value.e);
		System.out.println(fooB.printValue(value, true));
		
		Class c = Foo.class;
		Field f = c.getField("y");		
		Type t = f.getGenericType();
		ParameterizedType p = (ParameterizedType) t;		
		
		for (Type x : p.getActualTypeArguments() )
			System.out.println(x);
		
		MyRecord v2 = new MyRecord();
		v2.id = "ABC";
		v2.name = "Lamp";
		v2.x = 100;
		v2.y = 0;
		v2.z = 10;
		v2.linkToNext = null;		

		MyRecord v = new MyRecord();
		v.id = "KHF-IF 1234";
		v.name = "Storage box";
		v.x = 100;
		v.y = 0;
		v.z = 10;
		v.linkToNext = v2;
		
		
		Binding b = Bindings.getBinding(MyRecord.class);
		System.out.println( b.printValue(v, true) );
		System.out.println( );
		System.out.println( b.printValue(v, false ) );

		Binding bb = Bindings.getBinding(DataType.class);
//		System.out.println( );
//		System.out.println( bb.printValue(b.getDataType(), true ) );
		System.out.println( );
		System.out.println( b.getDataType().toString() );

//		String type = 
//		"type FEMMesh = {"+
//		"       topology : Topology,"+
//		"       data : MeshData[]"+
//		"     }"+			  
//		"type union Topology ="+ 
//		"        TriangleMesh {"+ 
//		"          vertexCount : Integer,"+
//		"	       faces : Integer[3][]"+
//		"	     }"+
//		"	     | PolygonMesh {"+
//		"	       vertexCount : Integer,"+
//		"	       faces : Integer[][]"+
//		"	     }"+			  
//		"type union MeshData = VertexData VectorArray"+
//		"	                  | FaceData VectorArray"+			 
//		"	    VectorArray = Double[..,..]";
//		DataType dt = DataTypes.translate(type);
//		System.out.println(dt);
		
*/
		
//		MyInterface obj = new MyInterface() {
//			public void hello() {
//				System.out.println("World");
//			}};
//		
//		MethodInterface mi = DataTypes.bindInterface(MyInterface.class, obj);
//		
//		DataType dt = DataTypes.getDataType( DataType.class );
//		System.out.println( dt );

//		System.out.println(String.format("%016x", Long.MAX_VALUE));
		
		
//		Pattern dp = Pattern.compile("\\d*(?:\\.\\d*)?(?:[eE]\\d++)?");
//		System.out.println(  dp.matcher("4").matches()  );
		
//		DataType dt = DataTypes.getDataTypeUnchecked(org.simantics.databoard.util.Range.class);
//		System.out.println(dt);
		
//		Pattern _ = Pattern.compile("(abu|dabu|)");
//		System.out.println( _.matcher("dabu").matches() );
//		System.out.println( _.matcher("abu").matches() );
//		System.out.println( _.matcher("gabu").matches() );
//		System.out.println( _.matcher("").matches() );
		
		//  [5..6]
//		Pattern p = Pattern.compile("(?:([\\(\\[])(\\d*(?:\\.\\d*)?(?:[eE]\\d++)?)\\.\\.(\\d*(?:\\.\\d*)?(?:[eE]\\d++)?)([\\)\\]])|([\\(\\[])(\\d*(?:\\.\\d*)?(?:[eE]\\d++)?)([\\)\\]])|)");
//		System.out.println( p.matcher("[0..6]").matches() );
//		System.out.println( p.matcher("[0..6.0]").matches() );
//		System.out.println( p.matcher("[0..6)").matches() );
//		System.out.println( p.matcher("[0..3").matches() );
//		System.out.println( p.matcher("[6]").matches() );
//		System.out.println( p.matcher("[.0]").matches() );
//		System.out.println( p.matcher("[2)").matches() );
//		System.out.println( p.matcher("[.2").matches() );
//		Matcher m = p.matcher("[0..6]");
//		System.out.println(m.matches());
//		System.out.println( m.group(2) );		
//		System.out.println( m.group(3) );
//		m = p.matcher("[6]");
//		System.out.println(m.matches());
//		System.out.println( m.group(6) );		
		
//		System.out.println(new Double(5).equals(new Integer(5)));
//		System.out.println( new Double("0.") );
//
//		Pattern p = Pattern.compile("\\d*(?:\\.\\d*)?(?:[eE]\\d++)?");
//		System.out.println("Fail Tests:");
//		System.out.println( p.matcher("243234e").matches() );
//		System.out.println( p.matcher("asdf").matches() );
//		System.out.println( p.matcher("35.43.").matches() );
//		System.out.println( p.matcher("e0").matches() );
//		System.out.println( p.matcher("").matches() );
//		System.out.println( p.matcher("12342a324").matches() );
//		System.out.println( p.matcher(".2432.").matches() );
//		System.out.println( p.matcher("22234e").matches() );
//		System.out.println( p.matcher("234234e4324e").matches() );
//		System.out.println( p.matcher(".").matches() );
//		System.out.println("Tests:");
//		System.out.println( p.matcher("0").matches() );
//		System.out.println( p.matcher(".0").matches() );
//		System.out.println( p.matcher("0.").matches() );
//		System.out.println( p.matcher(".").matches() );
//		System.out.println( p.matcher("0e0").matches() );
//		System.out.println( p.matcher("0E0").matches() );
//		System.out.println( p.matcher("2313123123123").matches() );
//		System.out.println( p.matcher(".2432423424323").matches() );
//		System.out.println( p.matcher(".2342343e1232312").matches() );
//		System.out.println( p.matcher("123123.123123e1231233").matches() );
//		System.out.println( p.matcher("123213").matches() );
		
//		System.out.println(new Double(0.5));
//		System.out.println(new Double(500000000000000.0));
//		System.out.println(new Double("5.0E0"));
		
//		InterfaceType it = DataTypes.getInterfaceType(HelloMBean.class);
//		System.out.println(it);
		
//		DataType dt = DataTypes.getDataType("{ x : Double, y : Double }");
		
		
		
//		Logger LOGGER = Logger.getLogger("Jotakin");
//		LOGGER.log(Level.INFO, "Info");
//		LOGGER.log(Level.SEVERE, "Error");
//		LOGGER.log(Level.WARNING, "Warning");
		
		

		
//	      MBeanServer mbs = ManagementFactory.getPlatformMBeanServer(); 
//	      ObjectName name = new ObjectName("opc.tcp://www.fi:type=Hello"); 
//	      ObjectName name2 = new ObjectName("opc.tcp://www.fi:type=Hello"); 
//	      Hello mbean = new Hello(); 
//	      mbs.registerMBean(mbean, name);
//	      mbs.registerMBean(mbean, name2);
//	      
//	      MBeanInfo info = mbs.getMBeanInfo(name);
//	      mbs.invoke(name, "sayHello", null, null);
//	 
//	      System.out.println("Waiting forever..."); 
//	      Thread.sleep(Long.MAX_VALUE); 		
		
		
//		ScheduledThreadPoolExecutor exe = new ScheduledThreadPoolExecutor( 4 );
//		Socket s2 = new Socket();
//		s2.setSoTimeout(1);
//		s2.connect(new InetSocketAddress("localhost", 4840));
//		System.out.println(s2.isConnected());
//		System.out.println(s2.isClosed());
//		s2.close();
//		System.out.println(s2.isConnected());
//		System.out.println(s2.isClosed());
//		
//		Executor e = new ThreadPoolExecutor(
//				0, 
//				Integer.MAX_VALUE,
//                3L, TimeUnit.SECONDS,
//                new LinkedBlockingQueue<Runnable>());
//		
//		e.execute(new Runnable() {
//			@Override
//			public void run() {
//				System.out.println("AA!!!");
//				throw new RuntimeException("AA!!!");
//			}});
//		
//		Thread.sleep(1000L);
//
//		e.execute(new Runnable() {
//			@Override
//			public void run() {
//				System.out.println("TOIMII VIEL!!!");
//				throw new RuntimeException("TOIMII VIEL!!");
//			}});
//		
//		
//		Serializer s = DoubleArrayBinding.INSTANCE.serializer(BinarySerializationFormat.INSTANCE);
//		s.serialize(new double[] {1,2,3,4});
//				
//		
//		File x = new File("<>");
//		System.out.println(x.toURI().toString());
//		System.out.println(x.toURI().toASCIIString());
//		
//		UnionType dt = (UnionType) DataTypes.getDataType(DataType.class);
//		System.out.println(dt.components.length);
//		
//		UnionBinding ub = (UnionBinding) DataTypes.getBinding(DataType.class);
//		System.out.println(ub.getComponentBindings().length);
	}
	
}
