/*******************************************************************************
 * Copyright (c) 2010- Association for Decentralized Information Management in
 * Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.tests;

import org.simantics.databoard.Bindings;
import org.simantics.databoard.binding.VariantBinding;
import org.simantics.databoard.binding.java.BooleanBindingDefault;
import org.simantics.databoard.binding.java.IntegerBindingDefault;
import org.simantics.databoard.binding.java.LongBindingDefault;
import org.simantics.databoard.binding.java.StringBindingDefault;
import org.simantics.databoard.bindingscheme.RandomValue;
import org.simantics.databoard.history.StringVariantBinding;
import org.simantics.databoard.type.VariantType;

import junit.framework.TestCase;

public class TestStringVariantBinding extends TestCase {

	public void testAll() throws Exception 
	{		
		RandomValue rv = new RandomValue();
		VariantBinding vb = StringVariantBinding.INSTANCE;
		VariantBinding gen = (VariantBinding) Bindings.getBinding( VariantType.INSTANCE );
		
		System.out.println( vb.create(5, IntegerBindingDefault.INSTANCE) );
		System.out.println( vb.create(5L, LongBindingDefault.INSTANCE) );
		System.out.println( vb.create("PA11_Valve/Pressure", StringBindingDefault.INSTANCE) );
		System.out.println( vb.create(true, BooleanBindingDefault.INSTANCE) );
		System.out.println( vb.create(false, BooleanBindingDefault.INSTANCE) );
		
		System.out.println( vb.getValue("I5") );
		System.out.println( vb.getValue("L5") );
		System.out.println( vb.getValue("SMoi") );
		System.out.println( vb.getValue("BAAE=") );
		
		for (int i=0; i<10000; i++) {
			
			int seed = i;
			rv.getRandom().setSeed(seed);
			rv.getRandom().nextLong();
			
			Object value = gen.accept(rv);
			
			String str = (String) Bindings.adaptUnchecked(value, gen, vb);
			Object value2 = Bindings.adaptUnchecked(str, vb, gen);
						
			System.out.println("Testcase: "+i);
			System.out.println("String: "+str);
			System.out.println("Value1: "+gen.printValueDefinition(value, true));
			System.out.println("Value2: "+gen.printValueDefinition(value2, true));
			System.out.println("Value3: "+vb.printValueDefinition(str, true));
			System.out.println("");
			
			assertTrue( gen.equals(value, value2) );
		}
	}
	
}

